mod add_remove;
mod assembly;
mod display;
mod outline;

use nalgebra::DVector;
use rustc_hash::FxHashSet;
use std::collections::BTreeSet;
use sycamore::prelude::*;

use add_remove::AddRemove;
use assembly::{Assembly, Constraint, Element};
use display::Display;
use outline::Outline;

#[derive(Clone)]
struct AppState {
    assembly: Assembly,
    selection: Signal<FxHashSet<usize>>
}

impl AppState {
    fn new() -> AppState {
        AppState {
            assembly: Assembly::new(),
            selection: create_signal(FxHashSet::default())
        }
    }
}

fn main() {
    sycamore::render(|| {
        let state = AppState::new();
        let assemb = &state.assembly;
        let _ = assemb.try_insert_element(
            Element {
                id: String::from("wing_a"),
                label: String::from("Wing A"),
                color: [1.00_f32, 0.25_f32, 0.00_f32],
                rep: DVector::<f64>::from_column_slice(&[0.5, 0.5, 0.0, 0.5, -0.25]),
                constraints: BTreeSet::default()
            }
        );
        let _ = assemb.try_insert_element(
            Element {
                id: String::from("wing_b"),
                label: String::from("Wing B"),
                color: [0.00_f32, 0.25_f32, 1.00_f32],
                rep: DVector::<f64>::from_column_slice(&[-0.5, -0.5, 0.0, 0.5, -0.25]),
                constraints: BTreeSet::default()
            }
        );
        let _ = assemb.try_insert_element(
            Element {
                id: String::from("central"),
                label: String::from("Central"),
                color: [0.75_f32, 0.75_f32, 0.75_f32],
                rep: DVector::<f64>::from_column_slice(&[0.0, 0.0, 0.0, 0.4, -0.625]),
                constraints: BTreeSet::default()
            }
        );
        assemb.insert_constraint(
            Constraint {
                args: (
                    assemb.elements_by_id.with(|elts_by_id| elts_by_id["wing_a"]),
                    assemb.elements_by_id.with(|elts_by_id| elts_by_id["wing_b"])
                ),
                rep: 0.5
            }
        );
        provide_context(state);
        
        view! {
            div(id="sidebar") {
                AddRemove {}
                Outline {}
            }
            Display {}
        }
    });
}